/*
 * Copyright (c) 2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision$
 * $LastChangedDate$
 * 
 */

package com.codedroids.oc.freemarker;

import java.util.Arrays;
import java.util.List;

import org.opencms.db.CmsUserSettings;
import org.opencms.file.CmsObject;
import org.opencms.file.CmsResource;
import org.opencms.file.CmsUser;
import org.opencms.i18n.CmsEncoder;
import org.opencms.i18n.CmsMessages;
import org.opencms.jsp.CmsJspTagProperty;
import org.opencms.main.CmsException;
import org.opencms.main.OpenCms;
import org.opencms.site.CmsSite;
import org.opencms.site.CmsSiteManager;
import org.opencms.staticexport.CmsLinkManager;

public class CmsUtils {
	
	// TODO: check if newer versions provide API get to the info provided by these hacks
	
	public static String label(CmsObject cmso, String label) 
	{
		CmsMessages messages = OpenCms.getWorkplaceManager().getMessages(cmso.getRequestContext().getLocale());
		return messages.key(label);
	}
	
    public static String link(CmsObject cmso, String link) 
	{
        if (link.indexOf(':') >= 0) {
            return OpenCms.getLinkManager().substituteLink(cmso, link);
        } else {
			CmsSite site = CmsSiteManager.getCurrentSite(cmso);
			String abs = CmsLinkManager.getAbsoluteUri(link, site.getUrl()+site.getSiteRoot());
            return OpenCms.getLinkManager().substituteLink(cmso, abs);
        }
    }
    
	public static String property(CmsObject cmso, String property, String action) throws CmsException {
		return property(cmso, property, action, null, false);
	}
	public static String property(CmsObject cmso, String property, String action, String defaultValue) throws CmsException {
		return property(cmso, property, action, defaultValue, false);
	}
	public static String property(CmsObject cmso, String property, String action, String defaultValue, boolean escapeHtml) throws CmsException
	{
		String value;
		
		// if action is not set use default
		if (action == null) {
			action = CmsJspTagProperty.ACTION_VALUES[0];
		}
		
		switch (CmsJspTagProperty.ACTION_VALUES_LIST.indexOf(action)) {
		case 0: // USE_URI
		case 1: // USE_PARENT
			// Read properties of parent (i.e. top requested) file
			value = cmso.readPropertyObject(
					cmso.getRequestContext().getUri(),
					property,
					false).getValue(defaultValue);
			break;
		case 2: // USE_SEARCH
		case 3: // USE_SEARCH_URI
		case 4: // USE_SEARCH_PARENT 
			// Try to find property on parent file and all parent folders
			value = cmso.readPropertyObject(
					cmso.getRequestContext().getUri(),
					property,
					true).getValue(defaultValue);
			break;
		case 5: // USE_ELEMENT_URI
		case 6: // USE_THIS
			// Read properties of this file            
			value = "??? property: \""+action+"\" ???"; //cmso.readPropertyObject(controller.getCurrentRequest().getElementUri(),property,false).getValue(defaultValue);
			break;
		case 7: // USE_SEARCH_ELEMENT_URI
		case 8: // USE_SEARCH_THIS
			// Try to find property on this file and all parent folders
			value = "??? property: \""+action+"\" ???"; //cmso.readPropertyObject(controller.getCurrentRequest().getElementUri(),property,true).getValue(defaultValue);
			break;
		default:
			// Read properties of the file named in the attribute            
			value = cmso.readPropertyObject(
					action/*link(cmso, action)*/,
					property,
					false).getValue(defaultValue);
		}
		if (escapeHtml) {
			value = CmsEncoder.escapeHtml(value);
		}
		return value;
	}
	
	// since these are private in CmsJspTagInfo
    /** Static array with allowed info property values. */
    private static final String[] SYSTEM_PROPERTIES = {"opencms.version", // 0
        "opencms.url", // 1
        "opencms.uri", // 2
        "opencms.webapp", // 3
        "opencms.webbasepath", // 4 
        "opencms.request.uri", // 5
        "opencms.request.element.uri", // 6
        "opencms.request.folder", // 7      
        "opencms.request.encoding", // 8
        "opencms.request.locale" // 9   
    };

    /** Array list of allowed property values for more convenient lookup. */
    private static final List SYSTEM_PROPERTIES_LIST = Arrays.asList(SYSTEM_PROPERTIES);
	
    public static String info(CmsObject cmso, String property) {

        if (property == null) {
             return "??? info: property was null ???";
        }
        String result = null;
        switch (SYSTEM_PROPERTIES_LIST.indexOf(property)) {
            case 0: // opencms.version
                result = OpenCms.getSystemInfo().getVersionName();
                break;
            case 1: // opencms.url
                result = "??? info: \""+property+"\" ???"; //req.getRequestURL().toString();
                break;
            case 2: // opencms.uri
                result = "??? info: \""+property+"\" ???"; //req.getRequestURI();
                break;
            case 3: // opencms.webapp
                result = OpenCms.getSystemInfo().getWebApplicationName();
                break;
            case 4: // opencms.webbasepath
                result = OpenCms.getSystemInfo().getWebApplicationRfsPath();
                break;
            case 5: // opencms.request.uri
                result = cmso.getRequestContext().getUri();
                break;
            case 6: // opencms.request.element.uri
                result = "??? info: \""+property+"\" ???"; //controller.getCurrentRequest().getElementUri();
                break;
            case 7: // opencms.request.folder
                result = CmsResource.getParentFolder(cmso.getRequestContext().getUri());
                break;
            case 8: // opencms.request.encoding
                result = cmso.getRequestContext().getEncoding();
                break;
            case 9: // opencms.request.locale
                result = cmso.getRequestContext().getLocale().toString();
                break;
            default:
                result = System.getProperty(property);
                if (result == null) {
                    return "??? info: \""+property+"\" ???";
                }
        }
        return result;
    }
    
    private static final String[] USER_PROPERTIES = {
        "name",
        "firstname",
        "lastname",
        "email",
        "street",
        "zip",
        "city",
        "description",
        "group",
        "currentgroup",
        "defaultgroup",
        "otherstuff"};

	/** Array list for fast lookup. */
	private static final List USER_PROPERTIES_LIST = Arrays.asList(USER_PROPERTIES);
	
	public static String user(CmsObject cmso, String property) {
		CmsUser user = cmso.getRequestContext().currentUser();
		
		if (property == null) {
			property = USER_PROPERTIES[0];
		}
		
		String result = null;
		switch (USER_PROPERTIES_LIST.indexOf(property)) {
		case 0: // name
			result = user.getName();
			break;
		case 1: // firstname
			result = user.getFirstname();
			break;
		case 2: // lastname
			result = user.getLastname();
			break;
		case 3: // email
			result = user.getEmail();
			break;
		case 4: // street
			result = user.getAddress();
			break;
		case 5: // zip
			result = (String)user.getAdditionalInfo(CmsUserSettings.ADDITIONAL_INFO_ZIPCODE);
			break;
//		case 6: // city
//			result = (String)user.getAdditionalInfo(CmsUserSettings.ADDITIONAL_INFO_TOWN);
//			break;
		case 7: // description
			result = user.getDescription();
			break;
		case 8: // group
		case 9: // currentgroup
		case 10: // defaultgroup
			result = "";
			break;
		case 11: // otherstuff
			result = "??? user: \""+property+"\"";
			break;
		default:
			result = "??? user: \""+property+"\"";
		}
		
		return result;
	}
}
